// AMcmd.cpp: implementation of the AMcmd class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AMcmd.h"
#include "AMcmdLogon.h"

#include <conio.h>


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AMcmd::AMcmd()
{
	m_argv = NULL;
	m_cmdList = NULL;
	m_argc = 0;
	m_nCmd = 0;
	m_batch = 0;
	m_parent = this;
}

AMcmd::~AMcmd()
{
	if (m_argv) free(m_argv);
}

int AMcmd::IsReadyForMore()
{
	return 1;
}

int AMcmd::Parse(TCHAR *buf)
{
	// m_batch = 0;

	// Are there any registered commands
	if (m_nCmd == 0)
	{
		Error(AMPERR_NO_COMMANDS);
		return AMPERR_NO_COMMANDS;
	}

	// NULL line
	if (!buf)
	{
		Error(AMPERR_BLANK_LINE);
		return AMPERR_BLANK_LINE;
	}
	
	// Split input buffer into m_argc, m_argv**
	SplitUp(buf);

	// Blank lines
	if (m_argc == 0)
	{
		Error(AMPERR_BLANK_LINE);
		return AMPERR_BLANK_LINE;
	}

	unsigned int i = 0; while (i < m_nCmd)
	{
		if (STRINGCOMPARE(m_argv[0], m_cmdList[i]->GetName(), 8))
		{
			i++;
		}
		else
		{
			m_isCancelled = 0;
			m_cmdList[i]->Parse(this);
			return 0; 
		}
	}

	Error (AMPERR_UNKNOWN_COMMAND);
	return AMPERR_UNKNOWN_COMMAND;
}

int AMcmd::CmdType(TCHAR*)
{
	return AM_LOGON;
}

void AMcmd::SplitUp(TCHAR *buf)
{
	unsigned int n = STRINGLENGTH(buf);
	int inside = 0;
	m_argc = 0;
	if (m_argv)
	{
		free(m_argv);
		m_argv = NULL;
	}

	unsigned int i = 0; 

	if (buf[0] == 0)
	{
		m_argc = 0;
		m_argv = NULL;
		return;
	}

	if (buf[0] != 34)
	{
		// We are entering a new token not in double quotes
		m_argv = (TCHAR**)realloc(m_argv, (++m_argc) * sizeof(TCHAR*));

		// argv[0] starts here
		m_argv[0] = buf;

		i++;
	}

	while (i < n)
	{
		// Inside - outside double quotes
		if (buf[i] == 34)
		{
			if (inside == 0)
			{
				// We are entering a new token enclosed in double quotes
				inside = 1;
				m_argv = (TCHAR**) realloc(m_argv, (++m_argc) * sizeof(TCHAR*));

				// argv[x] starts after double quote character
				m_argv[m_argc-1] = buf + i + 1;
			}
			else
			{
				// Check if next is also a double quote (ok because NULL at end)
				if (buf[i+1] == 34)
				{
					unsigned int j;

					// Skip one of the double quotes
					for (j = i; j < n; j++)
					{
						buf[j] = buf[j+1];
					}
					n--; // after loop so we get NULL termination moved as well
				}
				else
				{
					// We are getting out of double quotes, NULL terminate argv[x];
					inside = 0;
					buf[i] = 0;
				}
			}
		}
		else
		{
			if (buf[i] == VK_SPACE)
			{
				// We are entering a new token not in double quotes
				m_argv = (TCHAR**)realloc(m_argv, (++m_argc) * sizeof(TCHAR*));

				// argv[x] starts after blank
				m_argv[m_argc-1] = buf + i + 1;

				// NULL terminate old token
				buf[i] = 0;
			}
		}

		i++;
	}

}

void AMcmd::Error(unsigned int err)
{
	switch (err)
	{
	case AMPERR_BLANK_LINE:
		// PRINTTERM (_T("Blank line encountered\n"));
		break;
	case AMPERR_NO_COMMANDS:
		PRINTTERM (_T("No commands are defined\n"));
		break;
	case AMPERR_DONT_CALL_BASE:
		PRINTTERM (_T("Interpreter not implemented for this command\n"));
		break;
	case AMPERR_UNKNOWN_COMMAND:
		PRINTTERM (_T("Unknown command %s\n"), m_argv[0]);
		break;
	default:
		PRINTTERM (_T("Error %d in %s\n"), err);
		break;
	}
}

int AMcmd::RegisterCmd(AMcmd *cmd)
{
	if (!cmd->GetName()) return AMPERR_FATAL;

	m_cmdList = (AMcmd**) realloc(m_cmdList, sizeof(AMcmd*) * (m_nCmd+1));
	if (!m_cmdList) return AMPERR_FATAL;
	m_cmdList[m_nCmd] = cmd;

	m_nCmd++;
	return m_nCmd - 1;
}

TCHAR* AMcmd::GetName()
{
	return NULL; // _T("Base class has no name");
}

int AMcmd::Parse(AMcmd* dummy)
{
	return AMPERR_DONT_CALL_BASE;
}

TCHAR* AMcmd::PromptForString(TCHAR* prompt, TCHAR* def)
{
	// If we were cancelled earlier, just get out as fast as possible
	if (m_parent->m_isCancelled != 0) return NULL;
	
	// Missing parameters in batch means cancel
	if (m_parent->m_batch != 0)
	{
		PRINTTERM(_T("Parameters missing. Syntax is\n"));
		DisplayHelp();
		m_parent->m_isCancelled = 1;
	}

	TCHAR b[2048], *p;

	if (def)
	{
		STRINGPRINT(b, _T("%s (def=%s)> "), prompt, def);
	}
	else
	{
		STRINGPRINT(b, _T("%s> "), prompt);
	}

	PRINTTERM(b);

	if (!READTERM(b))
	{
		m_parent->m_isCancelled = 1;
		return NULL;
	}


	unsigned int i = (STRINGLENGTH(b)+1) * sizeof(TCHAR);

	if (i <= 1 && def != NULL)
	{
		i = STRINGLENGTH(def)+1;
		p = (TCHAR*) malloc(i);
		if (!p) return NULL;
		STRINGCOPY(p, def);
	}
	else
	{
		p = (TCHAR*) malloc(i);
		if (!p) return NULL;
		STRINGCOPY(p, b);
	}

	return p;
}

TCHAR* AMcmd::PromptForPasswordString(TCHAR* prompt, TCHAR* def)
{
	// If we were cancelled earlier, just get out as fast as possible
	if (m_parent->m_isCancelled != 0) return NULL;
	
	// Missing parameters in batch means cancel
	if (m_parent->m_batch != 0)
	{
		PRINTTERM(_T("Parameters missing. Syntax is\n"));
		DisplayHelp();
		m_parent->m_isCancelled = 1;
	}

	TCHAR b[2048], *p;
	unsigned int pos, maxpos;

	if (def)
	{
		STRINGPRINT(b, _T("%s (def=%s)> "), prompt, def);
	}
	else
	{
		STRINGPRINT(b, _T("%s> "), prompt);
	}

	PRINTTERM(b);

	TCHAR c;
	maxpos = pos = 0;
	while ((pos < 2048) && ((c=_getch()) != VK_RETURN))
	{
		AMBOOL useit = TRUE;

		if (c == VK_NUMPAD)
		{
			_putch(7);
			c =_getch();
			continue;
		}

		if (c == VK_BACK)
		{
			useit = FALSE;
			if (pos > 0)
			{
				pos--;
				_putch(VK_BACK);
				_putch(VK_SPACE);
				_putch(VK_BACK);
				continue;
			}
		}

		// Skip the traditional control characters
		if (c < VK_SPACE) useit = FALSE;

		// Echo to screen and put in buffer if applicable
		if (useit)
		{
			_putch(42);
			if (pos > maxpos) maxpos = pos;
			b[pos++] = c;
		}
		else
		{
			_putch(7);
		}
	}
	b[maxpos+1] = 0;
	PRINTTERM(_T("\n"));

	unsigned int i = (STRINGLENGTH(b)+1) * sizeof(TCHAR);

	p = (TCHAR*) malloc(i);
	if (!p) return NULL;

	STRINGCOPY(p, b);
	return p;
}

TCHAR* AMcmd::GetParam(unsigned int i)
{
	if (m_parent->m_isCancelled != 0) return NULL;
	unsigned int n;

	n = (STRINGLENGTH(m_argv[i])+1) * sizeof(TCHAR);
	if (i > m_argc) return NULL;

	TCHAR *p = (TCHAR*) malloc(n);
	if (!p) return NULL;

	STRINGCOPY(p, m_argv[i]);
	return p;
}

void AMcmd::FreeParam(TCHAR **p)
{
	if (*p)
	{

		free(*p);
		*p = NULL;
	}
}

int AMcmd::IsCancelled()
{
	return m_isCancelled;
}

unsigned int AMcmd::GetCmdCount()
{
	return m_nCmd;
}

AMcmd* AMcmd::GetCmd(unsigned int i)
{
	return m_cmdList[i];
}

void AMcmd::DisplayHelp()
{
	PRINTTERM(_T("No help for this command\n"));
}

void AMcmd::StringCat(TCHAR *t, TCHAR *f, unsigned int n)
{
	// Append precisely N characters from f to t. Fill with blanks or cut.
	// We assume that there is writable space and that n < 1024
	unsigned int i = 0;
	TCHAR c = 1;
	TCHAR *p = t;

	while(*p != 0)
	{
		if (++i > 1024) return; // just to make sure errors don't loop
		p++;
	}

	i = 0; while(i < n)
	{
		if (c != 32) c = *(f+i);
		if (c == 0) c = 32;
		*(p+i) = c;
		i++;
	}

	// Add blank and terminate with NULL
	*(p+i) = 32; i++;
	*(p+i) = 0;

}

void __stdcall AMcmd::WriteLog(TCHAR *txt, void *thys)
{
	PRINTTERM(txt);
}
