// AMcmdLogon.cpp: implementation of the AMcmdLogon class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AMcmdLogon.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AMcmdLogon::AMcmdLogon()
{
	m_parent = this;
	m_batch = 0;
}

AMcmdLogon::~AMcmdLogon()
{

}

AMcmdLogon::AMcmdLogon(AMcmd* parent)
{
	m_parent = parent;
}

int AMcmdLogon::Parse(AMcmd* parent)
{
	m_parent = parent;

	switch(parent->m_argc)
	{
	case 0:
		Error(AMPERR_FATAL);
		return AMPERR_FATAL;

	case 1:
		m_dsn=PromptForDataSource();
		m_uid=PromptForUser();
		m_pwd=PromptForPassword();
		break;

	case 2:
		m_dsn=parent->GetParam(1);
		m_uid=PromptForUser();
		m_pwd=PromptForPassword();
		break;

	case 3:
		m_dsn=parent->GetParam(1);
		m_uid=parent->GetParam(2);
		m_pwd=PromptForPassword();
		break;

	default:
		// Warning(AMPERR_EXCESS_PARAM);
		// fall through

	case 4:
		m_dsn=parent->GetParam(1);
		m_uid=parent->GetParam(2);
		m_pwd=parent->GetParam(3);
		break;
	}

	if (parent->IsCancelled() != 0)
	{
		return AMPERR_CANCELLED;
	}

	/*
	if (m_pwd)
	{
		PRINTTERM(_T("Logon %s %s %s\n"), m_dsn, m_uid, m_pwd);
	}
	else
	{
		PRINTTERM(_T("%s %s %s cancelled. No password\n"), GetName(), m_dsn, m_uid);
	}
	*/

	Execute();

	parent->FreeParam(&m_dsn);
	parent->FreeParam(&m_uid);
	parent->FreeParam(&m_pwd);
	return 0;
}

TCHAR* AMcmdLogon::GetName()
{
	return _T("LOGON");
}

TCHAR* AMcmdLogon::PromptForDataSource()
{
	return PromptForString(_T("ODBC data source"), _T("NetOp_Security_Evaluation"));
}

TCHAR* AMcmdLogon::PromptForUser()
{
	return PromptForString(_T("ODBC user name"), GETENV(_T("USERNAME")));
}

TCHAR* AMcmdLogon::PromptForPassword()
{
	return PromptForPasswordString(_T("ODBC password"), NULL);
}

void AMcmdLogon::DisplayHelp()
{
	PRINTTERM(_T("LOGON datasource username password\n"));
	PRINTTERM(_T("LOGON will log you on to the database\n"));
}

int AMcmdLogon::Execute()
{
	AMRETURN rc;

	// rc = theDbc.SetLogCallback((void*)Write, (void*)this);
	rc = theDbc.Logon(m_dsn, m_uid, m_pwd);

	if (rc == AM_SUCCESS || rc == AM_ERR_NO_TRANSACT)
	{
		PRINTTERM(_T("Logon ok\n"));
	}
	else
	{
		Error(rc);
	}

	return rc;
}

void AMcmdLogon::Error(int err)
{
	switch (err)
	{
	case AM_ERR_NULL_DSN:
		PRINTTERM (_T("Invalid data source name\n"));
		break;
	case AM_ERR_IS_ALREADY_LOGGED_ON:
		PRINTTERM (_T("You are already logged on\n"));
		break;
	case AM_ERR_INCONSISTENT_DATABASE:
		PRINTTERM (_T("The database is inconsistent\n"));
		break;
	default:
		PRINTTERM (_T("Error %d\n"), err);
		break;
	}
}

void __stdcall AMcmdLogon::Write(TCHAR *txt, void *thys)
{
	PRINTTERM(txt); 
	PRINTTERM(_T("\n"));
}
