#ifndef AMLIB_INCLUDED
#define AMLIB_INCLUDED
// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the AMLIB_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// AMLIB_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.

#ifdef WIN32

#define OS_WIN_NT 1
#include <stdio.h>

// The AMLIB API is supposed to be independent of DTL include files for its user
#ifdef AMLIB_EXPORTS
    #include <ntypes.h>
    #include <dtl2.h>
// But this cannot be done for internal apps. Pt the below is needed for AMconfig
#else
    #define DTL_NAME_GUEST_UNQUALI 0x06
    #define DTL_NAME_USERNAME      0x09
    #define DTL_NAME_DNS           0x0B
    #define DTL_NAME_HOSTID        0x0A  // 991207 Normal name= Host id converted to have one prefix byte.
    #define DBE_GUEST_LOCKED       89
    #define DBE_GUEST_PASSWORD_X   87
    #define DBE_GUEST_NO_EXIST     85
    #define DBE_PASS_OK_EXPIRED    100
#endif

#ifdef AMLIB_EXPORTS
    #define AMLIB_API __declspec(dllexport)
#else
    #define AMLIB_API __declspec(dllimport)
#endif

#include <tchar.h>
#define TCHARLEN _tcslen
#define MEMCOPY memcpy
#define STRINGCOPY _tcscpy
#define STRINGLENGTH _tcslen
#define PRINTSTRING _stprintf
#define READSTRING _stscanf
#define STRINGCAT _tcscat
#define SAMESTRING !_tcscmp
#define DIFFSTRING _tcscmp
#define STRINGFIND _tcschr

// The below two characters must be the same
#define AM_SEPARATOR ','
#define AM_SEPARATOR_T _T(",")

#ifdef UNICODE
// unicode definitions
#define SQL_C_TCHAR SQL_C_WCHAR
#define SQL_TCHAR SQL_WCHAR
#else
// not unicode
#define SQL_C_TCHAR SQL_C_CHAR
#define SQL_TCHAR SQL_CHAR
#endif

// Not win32
#else
#define TCHARLEN some_string_length
#define AMLIB_API __stdcall
#define STRINGCOPY some_function_name
#define PRINTSTRING some_function_name
#endif

#define AM_APPNAME _T("NetOp Security Manager")
#define AM_NOW NULL
#define AMBOOL BOOL
#define U32 unsigned long
#define AMRETURN U32

// Enum help contexts
#define IDH_LOGON 1

// Map ids
#define AM_MAPID_201 201
#define AM_MAPID_202 202
#define AM_MAPID_203 203
#define AM_MAPID_204 204
#define AM_MAPID_205 205
#define AM_MAPID_206 206
#define AM_MAPID_207 207
#define AM_MAPID_208 208
#define AM_MAPID_209 209
#define AM_MAPID_210 210
#define AM_MAPID_211 211
#define AM_MAPID_212 212
#define AM_MAPID_213 213
#define AM_MAPID_220 220
#define AM_MAPID_221 221
#define AM_MAPID_222 222
#define AM_MAPID_223 223
#define AM_MAPID_224 224
#define AM_MAPID_225 225
#define AM_MAPID_226 226

// Enum error codes
#define AM_NOT_IMPLEMENTED 0xEFEFEFEF
#define AM_ERR_NULL_NAME 0xFFFFFFFF
#define AM_SUCCESS 0
#define AM_ERR_WRITELOG 0xCC01
#define AM_ERR_NULL_DSN 0xCC02
#define AM_ERR_IS_ALREADY_LOGGED_ON 0xCC03
#define AM_ERR_NOT_LOGGED_ON 0xCC04
#define AM_ERR_INCONSISTENT_DATABASE 0xCC05
#define AM_ERR_NULL_DATE 0xCC07
#define AM_ERR_NULL_USER 0xCC08
#define AM_ERR_NULL_MODIFIED 0xCC09
#define AM_ERR_MODIFIED_BY_SOMEONE 0xCC010
#define AM_ERR_NO_MATCH 0xCC011
#define AM_ERR_LOCKED DBE_GUEST_LOCKED
#define AM_ERR_ILLEGAL_PARM 0xCC013
#define AM_NO_MORE_DATA 0xCC014
#define AM_ERR_NOMEM 0xCC015
#define AM_ERR_WRONG_TYPE 0xCC016
#define AM_ERR_NO_TRANSACT 0xCC017
#define AM_ERR_NO_DOMAIN_SERVER 0xCC018
#define AM_ERR_CANCELLED 0xCC019
#define AM_ERR_SEE_LOG 0xCC020
#define AM_ERR_WRONG_PASSWORD DBE_GUEST_PASSWORD
#define AM_ERR_NONCONFORMING_PASSWORD DBE_PASS_FORMAT
#define AM_ERR_NO_MD5_IN_DTL 0xCC023
#define AM_ERR_NO_ODBC30_SUPPORT 0xCC024

// Enum alarm severities
#define AM_ALARMTYPE_FATAL 0xBB01		// something requires immediate attention
#define AM_ALARMTYPE_SEVERE 0xBB02		// somebody tried something illegal or status of database is unwanted
#define AM_ALARMTYPE_INCONSISTENT 0xBB03// the database is now harmlessly inconsistent
#define AM_ALARMTYPE_STANDARD 0xBB04	// something harmless, but not so normal happened
#define AM_ALARMTYPE_INFO 0xBB05		// plain logging of interesting events

// Enum alarm events
#define AM_ALARM_BASE 0xAA00
#define AM_ALARM_LOGON_NO_PASSWORD AM_ALARM_BASE + 0x01
#define AM_ALARM_LOGON_USER_DISABLED_ON_WRONG_PASSWORDS_ERROR AM_ALARM_BASE + 0x02
#define AM_ALARM_LOGON_USER_DISABLED_ON_TIMEOUT_ERROR AM_ALARM_BASE + 0x03
#define AM_ALARM_TRANSACTION_NOT_SUPPORTED AM_ALARM_BASE + 0x04
#define AM_ALARM_ROLLBACK AM_ALARM_BASE + 0x05
#define AM_ALARM_CHANGE_PWD_REJECTED AM_ALARM_BASE + 0x06
#define AM_ALARM_CHANGE_PWD_REJECTED_UNKNOWN AM_ALARM_BASE + 0x07
#define AM_ALARM_CHANGE_PWD_OK AM_ALARM_BASE + 0x08
#define AM_ALARM_SCHEDULER_LOGON_ERROR AM_ALARM_BASE + 0x09
#define AM_ALARM_NOT_ALL_GROUPS_USED AM_ALARM_BASE + 0x0A  // We allow 20k test for a list of NT groups which an NT user i a member of - more groups are ignored
#define AM_ALARM_CANT_OPEN_LOGTEXTFILE AM_ALARM_BASE + 0x0B
#define AM_ALARM_CHANGE_PWD_REJECTED_DISABLED AM_ALARM_BASE + 0x0C
#define AM_ALARM_UNKNOWN_HOST AM_ALARM_BASE + 0x0D
#define AM_ALARM_CANT_DELETE_LOGRECORDS AM_ALARM_BASE + 0x0E
#define AM_ALARM_UNLISTED_SERVER_CONNECTING AM_ALARM_BASE + 0x0F
#define AM_ALARM_DATABASE_HAS_NO_GROUPNAME AM_ALARM_BASE + 0x10
#define AM_ALARM_CANT_WRITE_LOGRECORDS AM_ALARM_BASE + 0x11
#define AM_ALARM_LOGON_OK AM_ALARM_BASE + 0x12
#define AM_ALARM_LOGON_INCREMENT_WRONG_PASSWORDS_COUNTER_ERROR AM_ALARM_BASE + 0x13
#define AM_ALARM_LOGON_UPDATE_PWD_LAST_USED_ERROR AM_ALARM_BASE + 0x14
#define AM_ALARM_LOGON_TO_UNASSIGNED_HOST__ATTEMPTED AM_ALARM_BASE + 0x15
#define AM_ALARM_UNKNOWN_HOST_REJECTED AM_ALARM_BASE + 0x16
#define AM_ALARM_EVENTLOG_INSERT_ERROR AM_ALARM_BASE + 0x17
#define AM_ALARM_LOGTEXTFILE_EXISTS AM_ALARM_BASE + 0x18
#define AM_ALARM_CANT_DELETE_TODO_RECORD AM_ALARM_BASE + 0x19
#define AM_ALARM_CANT_EXEC_TODO_RECORD AM_ALARM_BASE + 0x20
#define AM_ALARM_ROLLBACK_FAILED AM_ALARM_BASE + 0x21

#define AM_ALARM_LOGON_CHANGE_PASSWORD DBE_PASS_OK_FORCE
#define AM_ALARM_LOGON_WRONG_PASSWORD DBE_GUEST_PASSWORD
#define AM_ALARM_LOGON_USER_DISABLED_ON_WRONG_PASSWORDS DBE_GUEST_LOGIN_X
#define AM_ALARM_LOGON_USER_DISABLED_ON_TIMEOUT DBE_GUEST_PASSWORD_X
#define AM_ALARM_LOGON_USER_IS_UNKNOWN DBE_GUEST_NO_EXIST
#define AM_ALARM_LOGON_USER_IS_DISABLED DBE_GUEST_LOCKED
#define AM_ALARM_CHANGE_PWD_REJECTED_REPEAT DBE_PASS_HORIZON

typedef void (__stdcall *AM_MSG_FUNC)(TCHAR* txt, void *thys);
typedef void (_cdecl *AM_ALARM_FUNC)(U32 severity, U32 type, TCHAR* txt, void *thys);

// Flagging which tables are present - columns are pt not checked
#define AM_TAB_DOMN		0x000000001
#define AM_TAB_DONE		0x000000002
#define AM_TAB_GRUP		0x000000004
#define AM_TAB_MAIN		0x000000008
#define AM_TAB_NTGR		0x000000010
#define AM_TAB_NTUS		0x000000020
#define AM_TAB_ROLE		0x000000040
#define AM_TAB_TODO		0x000000080
#define AM_TAB_USER		0x000000100
#define AM_TAB_USGR		0x000000200
#define AM_TAB_WKST		0x000000400
#define AM_TAB_GRUH		0x000000800
#define AM_TAB_HOGR		0x000001000
#define AM_TAB_HOST		0x000002000
#define AM_TAB_BATH		0x000004000
#define AM_TAB_POLI		0x000008000
#define AM_TAB_SERV		0x000010000
#define AM_TAB_CONN		0x000020000
#define AM_TAB_EVNT		0x000040000
#define AM_TAB_ALL		0x00007FFFF

// Patterns for encoding rights bits into U32
#define AM_ROL_CONF 0x000000003 // two bits integer, legal values 0,1,2
#define AM_ROL_CTRL 0x000000004
#define AM_ROL_KEYB 0x000000008
#define AM_ROL_LOCK 0x000000010
#define AM_ROL_BLNK 0x000000020
#define AM_ROL_BOOT 0x000000040
#define AM_ROL_CLIP 0x000000080
#define AM_ROL_SEND 0x000000100
#define AM_ROL_RECV 0x000000200
#define AM_ROL_PRNT 0x000000400
#define AM_ROL_CHAT 0x000000800
#define AM_ROL_AUDI 0x000001000
#define AM_ROL_RUNP 0x000002000

#define AM_CONFIRM_NEVER 0
#define AM_CONFIRM_ALWAYS 1
#define AM_CONFIRM_LOGGED 2

#define AM_CB_NONE 0
#define AM_CB_ROVING 1
#define AM_CB_FIXED 2

#define AM_COND_NONE 0
#define AM_LIKE 1
#define AM_EQUAL 2
#define AM_GT 3
#define AM_LT 4
#define AM_COND_LAST 5

// Enum the possible actions
#define AM_NONE 0
#define AM_INSERT 1
#define AM_DELETE 2
#define AM_UPDATE 3
#define AM_LOGON  4
#define AM_LOGOFF 5
#define AM_RESET  6
#define AM_ENABLE 7
#define AM_DISABLE 8
#define AM_ADD 9
#define AM_REMOVE 10
#define AM_CONNECT 11
#define AM_LOGEVENT 12
#define AM_SERVICE 13
#define AM_LASTCMD 14

#define AM_NOUSER _T("@,!:@,!:@,!:@,!:")

/*
#define DTL_NAME_USERNAME		          0x09  // 991207 Windows login user name
#define DTL_NAME_HOSTID			          0x0A  // 991207 Normal name= Host id converted to have one prefix byte.
#define DTL_NAME_DNS   			          0x0B  // 991210
*/

// Enum the types that can be in the right pane list view
#define AM_PREV 0
#define AM_ROLE 64
#define AM_NTUS DTL_NAME_USERNAME
#define AM_USER DTL_NAME_GUEST_UNQUALI
#define AM_GRUP 65
#define AM_NTGR 66
#define AM_WKST DTL_NAME_DNS
#define AM_DOMN 67
#define AM_MAIN 68
#define AM_POLI 69
#define AM_TODO 70
#define AM_TODO_BATCH 71
#define AM_LOG 72
#define AM_USGR 73
#define AM_BATH 74
#define AM_RIGH 75
#define AM_HOST DTL_NAME_HOSTID
#define AM_GRUH 77
#define AM_PWD 78
#define AM_ANYB 79
#define AM_ANYB_T _T("79 ")
#define AM_NTHO 80
#define AM_SERV 81
#define AM_SSGR 82
#define AM_CONN 83
#define AM_JOHN 84
#define AM_EVNT 85
#define AM_ALL 86

// Enum the policy names. Use names so database is readable
#define AM_POLICY_GRPNAM   _T("Security Server Group Name")
#define AM_POLICY_LOGDAYS  _T("Delete log entries older than")
#define AM_POLICY_PWDMINLN _T("NetOp Password Minimum Length")
#define AM_POLICY_RESTRICT _T("Enforce NetOp Password Control")
#define AM_POLICY_PWDHIST  _T("NetOp Password History Length")
#define AM_POLICY_PWDLIFE  _T("NetOp Password Lifetime")
#define AM_POLICY_PWDWRONG _T("Lock NetOp Password After Errors")
#define AM_POLICY_PWDUNUSE _T("Lock NetOp Password If Not Used")
#define AM_POLICY_GUESTTYP _T("Preferred Guest Type")
#define AM_POLICY_HOSTTYPE _T("Preferred Host Type")
#define AM_POLICY_SERVICE_DATE _T("Last Serviced")

#define AM_POLICY_GRPNAM_DEF   _T("NetOp")
#define AM_POLICY_LOGDAYS_DEF  _T("7")
#define AM_POLICY_PWDMINLN_DEF _T("8")
#define AM_POLICY_RESTRICT_DEF _T("Yes")
#define AM_POLICY_PWDHIST_DEF  _T("10")
#define AM_POLICY_PWDLIFE_DEF  _T("0")
#define AM_POLICY_PWDWRONG_DEF _T("3")
#define AM_POLICY_PWDUNUSE_DEF _T("90")
#define AM_POLICY_GUESTTYP_DEF _T("NT")
#define AM_POLICY_HOSTTYPE_DEF _T("NTuser if logged in")
#define AM_POLICY_SERVICE_DATE_DEF _T("1000-01-01")

#define AM_POLICY_LOGDAYS_ALT  _T("7")
#define AM_POLICY_PWDMINLN_ALT _T("8")
#define AM_POLICY_RESTRICT_ALT _T("Yes")
#define AM_POLICY_PWDHIST_ALT  _T("10")
#define AM_POLICY_PWDLIFE_ALT  _T("0")
#define AM_POLICY_PWDWRONG_ALT _T("3")
#define AM_POLICY_PWDUNUSE_ALT _T("90")
#define AM_POLICY_GUESTTYP_ALT _T("NetOp User")
#define AM_POLICY_HOSTTYPE_ALT _T("Workstation")

#define AM_POLICY_HOSTTYPE_ALT2 _T("NetOp HostID")
#define AM_FORCE_UPDATE _T("Force")

// Maximun length of SQL statement
#define STATEMENTLEN 2048

// Number of records to fetch at a time
#define AM_BULKSIZE 50

// Lengths of local C text buffers are one longer to hold the terminating NULL
#define AM_UNLEN 256
#define AM_PARAMLEN 256 // must be able to hold longest string
#define AM_DESCRLEN 65
#define AM_NAMLEN 33
#define AM_DBUSRLEN 65
#define AM_DATELEN 21
#define AM_PWDLEN 33
#define AM_CMDLEN 12 // must be at leat 1 bigger than sizeof(D3LCMD), which pt is 10 hardcoded
#define AM_CALLBACKLEN 256
#define AM_TXTLEN 161
#define STR_LEN 129  // Table name max lenght is 128 (taken from MS example code)

// ODBC-SQL definitions of types. 
// Hardcoded sizes because #define does not substitute inside double quotes
/* Please note that a database independent Char is limited to 255, a.o. by Access 2000
#ifdef UNICODE
#define AM_DEF_DBUSER _T("Char(128)")			// AM_DBUSRLEN
#define AM_DEF_NTUSER _T("Char(255)")			// AM_UNLEN
#define AM_DEF_NTGROUP _T("Char(255)")			// AM_UNLEN
#define AM_DEF_NETOPUSER _T("Char(128)")		// AM_NAMLEN
#define AM_DEF_NETOPGROUP _T("Char(128)")		// AM_NAMLEN
#define AM_DEF_NETOPPASSWORD _T("Char(128)")	// AM_PWDLEN
#define AM_DEF_DESCRIPTION _T("Char(128)")		// AM_DESCRLEN
#define AM_DEF_CALLBACK _T("Char(128)")			// AM_CALLBACKLEN
#define AM_DEF_DATETIME _T("Char(40)")			// AM_DATELEN
#define AM_DEF_INTEGER _T("Integer")			// U32 used
#define AM_DEF_PARAM _T("Char(255)")			// AM_PARAMLEN must be able to hold any logged string
#define AM_DEF_BOOL _T("Integer")				// U32 used
#define AM_DEF_SERIAL _T("Integer")				// U32 used
#define AM_DEF_WORKSTATION _T("Char(255)")		// AM_UNLEN
#define AM_DEF_ROLE _T("Char(128)")				// AM_NAMLEN
#define AM_DEF_DOMAIN _T("Char(255)")			// AM_UNLEN
#else
*/
#define AM_DEF_DBUSER _T("Char(64)")			// AM_DBUSRLEN
#define AM_DEF_NTUSER _T("Char(255)")			// AM_UNLEN
#define AM_DEF_NTGROUP _T("Char(255)")			// AM_UNLEN
#define AM_DEF_NETOPUSER _T("Char(32)")			// AM_NAMLEN
#define AM_DEF_NETOPHOST _T("Char(32)")			// AM_NAMLEN
#define AM_DEF_NETOPGROUP _T("Char(32)")		// AM_NAMLEN
#define AM_DEF_NETOPPASSWORD _T("Char(32)")		// AM_PWDLEN
#define AM_DEF_DESCRIPTION _T("Char(64)")		// AM_DESCRLEN
#define AM_DEF_CALLBACK _T("Char(255)")			// AM_CALLBACKLEN
#define AM_DEF_DATETIME _T("Char(20)")			// AM_DATELEN
#define AM_DEF_EVENT _T("Char(10)")				// AM_CMDLEN
#define AM_DEF_INTEGER _T("Integer")			// U32 used
#define AM_DEF_PARAM _T("Char(255)")			// AM_PARAMLEN must be able to hold any logged string
#define AM_DEF_BOOL _T("Integer")				// U32 used
#define AM_DEF_SERIAL _T("Integer")				// U32 used
#define AM_DEF_WORKSTATION _T("Char(255)")		// AM_UNLEN
#define AM_DEF_POLI _T("Char(32)")				// AM_NAMLEN
#define AM_DEF_ROLE _T("Char(32)")				// AM_NAMLEN
#define AM_DEF_DOMAIN _T("Char(255)")			// AM_UNLEN
#define AM_DEF_AKEY _T("Char(32)")				// AM_NAMLEN
#define AM_DEF_TEXT _T("Char(160)")				// AM_TXTLEN

// #endif

// All table definitions start with DW so you hopefully kan mix them into an existing database
#define AMTABNAM_ROLE      _T("DWROLE")
#define AMCOLNUM_ROLE 20
#define AMCOLNUM_ROLE_RIGHTS 13
#define AMCOLNAM_ROLE_ID   _T("ID")
#define AMCOLNAM_ROLE_NAME _T("RoleName")
#define AMCOLNAM_ROLE_DESC _T("Description")
#define AMCOLNAM_ROLE_CTRL _T("Rctl")
#define AMCOLNAM_ROLE_KEYB _T("Keyb")
#define AMCOLNAM_ROLE_BLNK _T("Blnk")
#define AMCOLNAM_ROLE_LOCK _T("Lock")
#define AMCOLNAM_ROLE_BOOT _T("Boot")
#define AMCOLNAM_ROLE_CLIP _T("Clip")
#define AMCOLNAM_ROLE_SEND _T("Send")
#define AMCOLNAM_ROLE_RECV _T("Recv")
#define AMCOLNAM_ROLE_PRNT _T("Prnt")
#define AMCOLNAM_ROLE_CHAT _T("Chat")
#define AMCOLNAM_ROLE_AUDI _T("Audi")
#define AMCOLNAM_ROLE_CONF _T("Conf")
#define AMCOLNAM_ROLE_RUNP _T("RunP")

#define AMSUBITM_ROLE_NAME 0
#define AMSUBITM_ROLE_CTRL 1
#define AMSUBITM_ROLE_KEYB 2
#define AMSUBITM_ROLE_BLNK 3
#define AMSUBITM_ROLE_LOCK 4
#define AMSUBITM_ROLE_BOOT 5
#define AMSUBITM_ROLE_CLIP 6
#define AMSUBITM_ROLE_SEND 7
#define AMSUBITM_ROLE_RECV 8
#define AMSUBITM_ROLE_CHAT 9
#define AMSUBITM_ROLE_AUDI 10
#define AMSUBITM_ROLE_PRNT 11
#define AMSUBITM_ROLE_RUNP 12
#define AMSUBITM_ROLE_CONF 13
#define AMSUBITM_ROLE_DESC 14
#define AMSUBITM_ROLE_CREA 15
#define AMSUBITM_ROLE_CRBY 16
#define AMSUBITM_ROLE_MODI 17
#define AMSUBITM_ROLE_MOBY 18
#define AMSUBITM_ROLE_ID 19 

#define AMTABNAM_USER      _T("DWUSER")
#define AMCOLNUM_USER 26
#define AMCOLNAM_USER_NAME _T("UserName")
#define AMCOLNAM_USER_DESC _T("Description")
#define AMCOLNAM_USER_ID   _T("ID")
#define AMCOLNAM_USER_ENAB _T("Enabled")
#define AMCOLNAM_USER_PWD  _T("Password")
#define AMCOLNAM_USER_PWDU _T("PwdUsed")
#define AMCOLNAM_USER_PWDC _T("PwdChanged")
#define AMCOLNAM_USER_PWDW _T("PwdWrong")
#define AMCOLNAM_USER_PWDN _T("PwdNum")
#define AMCOLNAM_USER_PWD0 _T("Pwd0")
#define AMCOLNAM_USER_PWD1 _T("Pwd1")
#define AMCOLNAM_USER_PWD2 _T("Pwd2")
#define AMCOLNAM_USER_PWD3 _T("Pwd3")
#define AMCOLNAM_USER_PWD4 _T("Pwd4")
#define AMCOLNAM_USER_PWD5 _T("Pwd5")
#define AMCOLNAM_USER_PWD6 _T("Pwd6")
#define AMCOLNAM_USER_PWD7 _T("Pwd7")
#define AMCOLNAM_USER_PWD8 _T("Pwd8")
#define AMCOLNAM_USER_PWD9 _T("Pwd9")
#define AMCOLNAM_USER_STAT _T("ForceChange")
#define AMCOLNAM_USER_CABK _T("Callback")
#define AMCOLNAM_USER_CBMO _T("Mode")

#define AMTABNAM_HOST      _T("DWHOST")
#define AMCOLNUM_HOST 8
#define AMCOLNAM_HOST_NAME _T("HostName")
#define AMCOLNAM_HOST_DESC _T("Description")
#define AMCOLNAM_HOST_ID   _T("ID")
#define AMCOLNAM_HOST_ENAB _T("Enabled")

#define AMTABNAM_GRUP      _T("DWGRUP")
#define AMCOLNUM_GRUP 8
#define AMCOLNAM_GRUP_NAME _T("GroupName")
#define AMCOLNAM_GRUP_ENAB _T("Enabled")
#define AMCOLNAM_GRUP_DESC _T("Description")
#define AMCOLNAM_GRUP_ID   _T("ID")

#define AMTABNAM_GRUH      _T("DWGRUH")
#define AMCOLNUM_GRUH AMCOLNUM_GRUP
#define AMCOLNAM_GRUH_NAME AMCOLNAM_GRUP_NAME
#define AMCOLNAM_GRUH_ENAB AMCOLNAM_GRUP_ENAB
#define AMCOLNAM_GRUH_DESC AMCOLNAM_GRUP_DESC
#define AMCOLNAM_GRUH_ID   AMCOLNAM_GRUP_ID

#define AMTABNAM_USGR      _T("DWUSGR")
#define AMCOLNUM_USGR 6
#define AMCOLNAM_USGR_USID _T("UsrID")
#define AMCOLNAM_USGR_GRID _T("GrpId")

#define AMTABNAM_HOGR      _T("DWHOGR")
#define AMCOLNUM_HOGR 6
#define AMCOLNAM_HOGR_USID _T("HostID")
#define AMCOLNAM_HOGR_GRID _T("GrpId")

#define AMTABNAM_WKST      _T("DWWKST")
#define AMCOLNUM_WKST 8
#define AMCOLNAM_WKST_ID   _T("ID")
#define AMCOLNAM_WKST_NAME _T("ComputerName")
#define AMCOLNAM_WKST_DOMN _T("Domain")
#define AMCOLNAM_WKST_ENAB _T("Enabled")

#define AMTABNAM_DOMN      _T("DWDOMN")
#define AMCOLNUM_DOMN 7
#define AMCOLNAM_DOMN_ID   _T("ID")
#define AMCOLNAM_DOMN_NAME _T("DomainName")
#define AMCOLNAM_DOMN_ENAB _T("Enabled")

#define AMTABNAM_SERV      _T("DWSERV")
#define AMCOLNUM_SERV 8
#define AMCOLNAM_SERV_NAME _T("ServerName")
#define AMCOLNAM_SERV_IRUN _T("IsRunning")
#define AMCOLNAM_SERV_TYPE _T("ServerType")
#define AMCOLNAM_SERV_AKEY _T("ASkey")

#define AMTABNAM_NTUS      _T("DWNTUS")
#define AMCOLNUM_NTUS 9
#define AMCOLNAM_NTUS_ID   _T("ID")
#define AMCOLNAM_NTUS_RID  _T("RID")
#define AMCOLNAM_NTUS_NAME _T("UserName")
#define AMCOLNAM_NTUS_DOMN _T("Domain")
#define AMCOLNAM_NTUS_ENAB _T("Enabled")

#define AMTABNAM_NTGR      _T("DWNTGR")
#define AMCOLNUM_NTGR 9
#define AMCOLNAM_NTGR_ID   _T("ID")
#define AMCOLNAM_NTGR_RID  _T("RID")
#define AMCOLNAM_NTGR_NAME _T("GroupName")
#define AMCOLNAM_NTGR_DOMN _T("Domain")
#define AMCOLNAM_NTGR_ENAB _T("Enabled")

#define AMTABNAM_MAIN      _T("DWMAIN")
#define AMCOLNUM_MAIN 11
#define AMCOLNAM_MAIN_ID   _T("ID")
#define AMCOLNAM_MAIN_GUID _T("GuestID")
#define AMCOLNAM_MAIN_GUTY _T("GuestType")
#define AMCOLNAM_MAIN_HOID _T("HostID")
#define AMCOLNAM_MAIN_HOTY _T("HostType")
#define AMCOLNAM_MAIN_ROID _T("RoleID")
#define AMCOLNAM_MAIN_ENAB _T("Enabled")

#define AMTABNAM_CONN      _T("DWCONN")
#define AMCOLNUM_CONN 4
#define AMCOLNAM_CONN_GUST _T("Guest")
#define AMCOLNAM_CONN_HOST _T("Host")
#define AMCOLNAM_CONN_TYPE _T("SessionType")
#define AMCOLNAM_CONN_STAR _T("Started")

#define AMTABNAM_TODO      _T("DWTODO")
#define AMCOLNUM_TODO 10
#define AMCOLNAM_TODO_ID   _T("ID")
#define AMCOLNAM_TODO_BATH _T("JobID")
#define AMCOLNAM_TODO_EXEC _T("ExecuteAt")
#define AMCOLNAM_TODO_ACTI _T("Action")
#define AMCOLNAM_TODO_OPAN _T("Operand")
#define AMCOLNAM_TODO_OPER _T("Operator")
#define AMCOLNAM_TODO_P1   _T("P1")
#define AMCOLNAM_TODO_P2   _T("P2")
//#define AMCOLNAM_TODO_P3   _T("P3")

#define AMTABNAM_DONE      _T("DWDONE")
#define AMCOLNUM_DONE 8
#define AMCOLNAM_DONE_ID   _T("ID")
#define AMCOLNAM_DONE_STAT _T("Status")
#define AMCOLNAM_DONE_ACTI _T("Action")
#define AMCOLNAM_DONE_OPAN _T("Operand")
#define AMCOLNAM_DONE_OPER _T("Operator")
#define AMCOLNAM_DONE_P1   _T("P1")
// #define AMCOLNAM_DONE_P2   _T("P2")
// #define AMCOLNAM_DONE_P3   _T("P3")
		
#define AMTABNAM_EVNT      _T("DWEVNT")
#define AMCOLNUM_EVNT 8
#define AMCOLNAM_EVNT_ID   _T("ID")
#define AMCOLNAM_EVNT_DATE _T("Created")
#define AMCOLNAM_EVNT_EVNT _T("EventType")
#define AMCOLNAM_EVNT_ERRD _T("DtlError")
#define AMCOLNAM_EVNT_ERRP _T("ProtocolError")
#define AMCOLNAM_EVNT_HOST _T("Host")
#define AMCOLNAM_EVNT_SERI _T("SerialNo")
#define AMCOLNAM_EVNT_DESC _T("Description")

#define AMTABNAM_BATH      _T("DWBATH")
#define AMCOLNUM_BATH 12
#define AMCOLNAM_BATH_ID   _T("ID")
#define AMCOLNAM_BATH_DESC _T("Description")
#define AMCOLNAM_BATH_TYPE _T("Category")
#define AMCOLNAM_BATH_GRID _T("GroupID")
#define AMCOLNAM_BATH_DOMN _T("Domain")
#define AMCOLNAM_BATH_STIM _T("StartTime")
#define AMCOLNAM_BATH_ETIM _T("EndTime")
#define AMCOLNAM_BATH_FLAG _T("Flags")

#define AMTABNAM_POLI      _T("DWPOLI")
#define AMCOLNUM_POLI 2
#define AMCOLNAM_POLI_NAME _T("Parameter")
#define AMCOLNAM_POLI_VALU _T("Setting")

// Common columns
#define AMCOLNAM_CREA _T("Created")
#define AMCOLNAM_CRBY _T("CreatedBy")
#define AMCOLNAM_MODI _T("Modified")
#define AMCOLNAM_MOBY _T("ModifiedBy")

#define AM_FIXED_ROLE _T("Fixed role")
#define AM_PERMANENT_ROLE _T("Role can be modified, but not deleted")

#define AM_NO_PASSWORD     _T("00000000000000000000000000000000")
#define AM_ERASE_TEXT      _T("00000000000000000000000000000000") \
                           _T("00000000000000000000000000000000")

#define AMSQL_PTHS _T("(")
#define AMSQL_PTHE _T(") ")
#define AMSQL_DOT _T(".")
#define AMSQL_COMMA _T(",")
#define AMSQL_BLANK _T(" ")
#define AMSQL_EQUAL _T("=")
#define AMSQL_QUOTE _T("'")
#define AMSQL_IN _T("IN ")
#define AMSQL_IN_STRINGLIST AMSQL_IN _T("(%s) ")
#define AMSQL_AND _T("AND ")
#define AMSQL_FROM _T("FROM ")
#define AMSQL_WHERE _T("WHERE ")
#define AMSQL_UNION _T("UNION ")
#define AMSQL_SELECT _T("SELECT ")
#define AMSQL_DELETE _T("DELETE ")
#define AMSQL_DISTINCT _T("DISTINCT ")
#define AMSQL_ALTER _T("ALTER ")
#define AMSQL_TABLE _T("TABLE ")
#define AMSQL_ADD _T("ADD ")
#define AMSQL_COLUMN _T("COLUMN ")
#define AMSQL_SET _T("SET ")
#define AMSQL_UPDATE _T("UPDATE ")
#define AMSQL_DEFAULT _T("DEFAULT ")
#define AMSQL_IS_ENABLED _T(">0 ")
#define AMSQL_READ_INTEGER _T("=%d ")
#define AMSQL_READ_STRING _T("='%s'")

#define AM_DEF_COMMON_CREATED AMCOLNAM_CREA AMSQL_BLANK AM_DEF_DATETIME AMSQL_COMMA AMCOLNAM_CRBY AMSQL_BLANK AM_DEF_DBUSER
#define AM_DEF_COMMON_MODIFIED AMCOLNAM_MODI AMSQL_BLANK AM_DEF_DATETIME AMSQL_COMMA AMCOLNAM_MOBY AMSQL_BLANK AM_DEF_DBUSER
#define AM_DEF_COMMON AM_DEF_COMMON_CREATED AMSQL_COMMA AM_DEF_COMMON_MODIFIED

#define AMSQL_SELECT_RIGHTS \
	AMSQL_SELECT AMSQL_DISTINCT \
	AMTABNAM_ROLE AMSQL_DOT AMCOLNAM_ROLE_CTRL AMSQL_COMMA \
	AMTABNAM_ROLE AMSQL_DOT AMCOLNAM_ROLE_KEYB AMSQL_COMMA \
	AMTABNAM_ROLE AMSQL_DOT AMCOLNAM_ROLE_BLNK AMSQL_COMMA \
	AMTABNAM_ROLE AMSQL_DOT AMCOLNAM_ROLE_LOCK AMSQL_COMMA \
	AMTABNAM_ROLE AMSQL_DOT AMCOLNAM_ROLE_BOOT AMSQL_COMMA \
	AMTABNAM_ROLE AMSQL_DOT AMCOLNAM_ROLE_CLIP AMSQL_COMMA \
	AMTABNAM_ROLE AMSQL_DOT AMCOLNAM_ROLE_SEND AMSQL_COMMA \
	AMTABNAM_ROLE AMSQL_DOT AMCOLNAM_ROLE_RECV AMSQL_COMMA \
	AMTABNAM_ROLE AMSQL_DOT AMCOLNAM_ROLE_PRNT AMSQL_COMMA \
	AMTABNAM_ROLE AMSQL_DOT AMCOLNAM_ROLE_CHAT AMSQL_COMMA \
	AMTABNAM_ROLE AMSQL_DOT AMCOLNAM_ROLE_AUDI AMSQL_COMMA \
	AMTABNAM_ROLE AMSQL_DOT AMCOLNAM_ROLE_RUNP AMSQL_COMMA \
	AMTABNAM_ROLE AMSQL_DOT AMCOLNAM_ROLE_CONF \
	AMSQL_BLANK

#define AMSQL_COMMTABS \
	AMSQL_FROM AMTABNAM_ROLE AMSQL_COMMA AMTABNAM_MAIN

#define AMSQL_FROM_HOID \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_HOST AMSQL_BLANK

#define AMSQL_FROM_USER \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_USER AMSQL_BLANK

#define AMSQL_FROM_NTUS \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_NTUS AMSQL_BLANK

#define AMSQL_FROM_NTGR \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_NTGR AMSQL_BLANK

#define AMSQL_FROM_NTGR_NTUS \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_NTUS AMSQL_COMMA AMTABNAM_NTGR AMSQL_BLANK

#define AMSQL_FROM_WKST \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_DOMN AMSQL_COMMA AMTABNAM_WKST AMSQL_BLANK

#define AMSQL_FROM_DOMN \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_DOMN AMSQL_BLANK

#define AMSQL_FROM_USGR \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_USER AMSQL_COMMA AMTABNAM_USGR AMSQL_COMMA AMTABNAM_GRUP AMSQL_BLANK

#define AMSQL_FROM_HOGR \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_HOST AMSQL_COMMA AMTABNAM_HOGR AMSQL_COMMA AMTABNAM_GRUH AMSQL_BLANK

#define AMSQL_FROM_USER_HOID \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_USER AMSQL_COMMA AMTABNAM_HOST AMSQL_BLANK

#define AMSQL_FROM_USER_HOGR \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_USER AMSQL_COMMA AMTABNAM_HOST AMSQL_COMMA AMTABNAM_HOGR AMSQL_COMMA \
	AMTABNAM_GRUH AMSQL_BLANK

#define AMSQL_FROM_USGR_HOID \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_USER AMSQL_COMMA AMTABNAM_HOST AMSQL_COMMA AMTABNAM_USGR AMSQL_COMMA \
	AMTABNAM_GRUP AMSQL_BLANK

#define AMSQL_FROM_USGR_DOMN \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_USER AMSQL_COMMA AMTABNAM_USGR AMSQL_COMMA AMTABNAM_GRUP AMSQL_COMMA \
	AMTABNAM_DOMN AMSQL_BLANK

#define AMSQL_FROM_USER_DOMN \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_USER AMSQL_COMMA AMTABNAM_DOMN AMSQL_BLANK

#define AMSQL_FROM_USGR_WKST \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_USER AMSQL_COMMA AMTABNAM_USGR AMSQL_COMMA AMTABNAM_GRUP AMSQL_COMMA \
	AMTABNAM_DOMN AMSQL_COMMA AMTABNAM_WKST AMSQL_BLANK

#define AMSQL_FROM_USER_WKST \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_USER AMSQL_COMMA AMTABNAM_WKST AMSQL_COMMA AMTABNAM_DOMN AMSQL_BLANK

#define AMSQL_FROM_USGR_HOGR \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_USER AMSQL_COMMA AMTABNAM_HOST AMSQL_COMMA AMTABNAM_USGR AMSQL_COMMA \
	AMTABNAM_GRUP AMSQL_COMMA AMTABNAM_HOGR AMSQL_COMMA AMTABNAM_GRUH AMSQL_BLANK

#define AMSQL_FROM_USGR_NTUS \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_USER AMSQL_COMMA AMTABNAM_GRUP AMSQL_COMMA AMTABNAM_USGR AMSQL_COMMA \
	AMTABNAM_NTUS AMSQL_BLANK

#define AMSQL_FROM_NTGR_DOMN \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_DOMN AMSQL_COMMA AMTABNAM_NTGR AMSQL_BLANK

#define AMSQL_FROM_NTGR_WKST \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_DOMN AMSQL_COMMA AMTABNAM_WKST AMSQL_COMMA AMTABNAM_NTGR AMSQL_BLANK

#define AMSQL_FROM_NTGR_HOID \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_NTGR AMSQL_COMMA AMTABNAM_HOST AMSQL_BLANK

#define AMSQL_FROM_NTGR_HOGR \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_NTGR AMSQL_COMMA AMTABNAM_HOST AMSQL_COMMA AMTABNAM_HOGR AMSQL_COMMA \
	AMTABNAM_GRUH AMSQL_BLANK

#define AMSQL_FROM_NTUS_HOID \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_NTUS AMSQL_COMMA AMTABNAM_HOST AMSQL_BLANK

#define AMSQL_FROM_NTUS_HOGR \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_NTUS AMSQL_COMMA AMTABNAM_HOST AMSQL_COMMA AMTABNAM_HOGR AMSQL_COMMA \
	AMTABNAM_GRUH AMSQL_BLANK

#define AMSQL_FROM_NTUS_DOMN \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_DOMN AMSQL_COMMA AMTABNAM_NTUS AMSQL_BLANK

#define AMSQL_FROM_NTUS_WKST \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_DOMN AMSQL_COMMA AMTABNAM_WKST AMSQL_COMMA AMTABNAM_NTUS AMSQL_BLANK

#define AMSQL_FROM_USER_NTUS \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_USER AMSQL_COMMA AMTABNAM_NTUS AMSQL_BLANK

#define AMSQL_FROM_USER_NTGR \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_USER AMSQL_COMMA AMTABNAM_NTGR AMSQL_COMMA AMTABNAM_NTUS AMSQL_BLANK

#define AMSQL_FROM_USGR_NTGR \
	AMSQL_COMMTABS AMSQL_COMMA AMTABNAM_USER AMSQL_COMMA AMTABNAM_GRUP AMSQL_COMMA AMTABNAM_USGR AMSQL_COMMA \
	AMTABNAM_NTUS AMSQL_COMMA AMTABNAM_NTGR AMSQL_BLANK

#define AMSQL_WO \
	AMSQL_WHERE \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_ENAB AMSQL_IS_ENABLED AMSQL_AND \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_ROID AMSQL_EQUAL \
	AMTABNAM_ROLE AMSQL_DOT AMCOLNAM_ROLE_ID AMSQL_BLANK AMSQL_AND

#define AMSQL_IN_NTGR \
	AMSQL_IN AMSQL_PTHS AMSQL_SELECT \
	AMCOLNAM_NTGR_ID AMSQL_BLANK AMSQL_FROM AMSQL_BLANK AMTABNAM_NTGR AMSQL_BLANK AMSQL_WHERE \
	AMCOLNAM_NTGR_ENAB AMSQL_IS_ENABLED AMSQL_AND \
	AMCOLNAM_NTGR_NAME AMSQL_BLANK AMSQL_IN_STRINGLIST AMSQL_AND \
	AMCOLNAM_NTGR_DOMN AMSQL_READ_STRING AMSQL_PTHE

#define AMSQL_IN_NTUS \
	AMSQL_IN AMSQL_PTHS AMSQL_SELECT \
	AMCOLNAM_NTUS_ID AMSQL_BLANK AMSQL_FROM AMSQL_BLANK AMTABNAM_NTUS AMSQL_BLANK AMSQL_WHERE \
	AMCOLNAM_NTUS_ENAB AMSQL_IS_ENABLED AMSQL_AND \
	AMCOLNAM_NTUS_NAME AMSQL_READ_STRING AMSQL_BLANK AMSQL_AND \
	AMCOLNAM_NTUS_DOMN AMSQL_READ_STRING AMSQL_PTHE

#define AMSQL_GUEST_NTGR \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_GUTY AMSQL_READ_INTEGER AMSQL_AND \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_GUID AMSQL_BLANK \
	AMSQL_IN_NTGR

#define AMSQL_HOST_NTGR \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_HOTY AMSQL_READ_INTEGER AMSQL_AND \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_HOID AMSQL_BLANK \
	AMSQL_IN_NTGR

#define AMSQL_GUEST_NTUS \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_GUTY AMSQL_READ_INTEGER AMSQL_AND \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_GUID AMSQL_BLANK \
	AMSQL_IN_NTUS

#define AMSQL_HOST_NTUS \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_HOTY AMSQL_READ_INTEGER AMSQL_AND \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_HOID AMSQL_BLANK \
	AMSQL_IN_NTUS

#define AMSQL_HOST_DOMN \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_HOTY AMSQL_READ_INTEGER AMSQL_AND \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_HOID AMSQL_EQUAL \
	AMTABNAM_DOMN AMSQL_DOT AMCOLNAM_DOMN_ID AMSQL_BLANK AMSQL_AND \
	AMTABNAM_DOMN AMSQL_DOT AMCOLNAM_DOMN_ENAB AMSQL_IS_ENABLED AMSQL_AND \
	AMTABNAM_DOMN AMSQL_DOT AMCOLNAM_DOMN_NAME AMSQL_READ_STRING AMSQL_BLANK

#define AMSQL_HOST_WKST \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_HOTY AMSQL_READ_INTEGER AMSQL_AND \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_HOID AMSQL_EQUAL \
	AMTABNAM_WKST AMSQL_DOT AMCOLNAM_WKST_ID AMSQL_BLANK AMSQL_AND \
	AMTABNAM_WKST AMSQL_DOT AMCOLNAM_WKST_ENAB AMSQL_IS_ENABLED AMSQL_AND \
	AMTABNAM_WKST AMSQL_DOT AMCOLNAM_WKST_NAME AMSQL_READ_STRING AMSQL_BLANK AMSQL_AND \
	AMTABNAM_WKST AMSQL_DOT AMCOLNAM_WKST_DOMN AMSQL_READ_STRING AMSQL_BLANK

#define AMSQL_HOST_GRUH \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_HOTY AMSQL_READ_INTEGER AMSQL_AND \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_HOID AMSQL_EQUAL \
	AMTABNAM_GRUH AMSQL_DOT AMCOLNAM_GRUH_ID AMSQL_BLANK AMSQL_AND \
	AMTABNAM_GRUH AMSQL_DOT AMCOLNAM_GRUH_ENAB AMSQL_IS_ENABLED AMSQL_AND \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_HOID AMSQL_BLANK AMSQL_IN AMSQL_PTHS \
	AMSQL_SELECT AMTABNAM_HOGR AMSQL_DOT AMCOLNAM_HOGR_GRID AMSQL_BLANK AMSQL_FROM\
	AMTABNAM_HOGR AMSQL_COMMA AMTABNAM_HOST AMSQL_BLANK AMSQL_WHERE \
	AMTABNAM_HOGR AMSQL_DOT AMCOLNAM_HOGR_USID AMSQL_EQUAL \
	AMTABNAM_HOST AMSQL_DOT AMCOLNAM_HOST_ID AMSQL_BLANK AMSQL_AND \
	AMTABNAM_HOST AMSQL_DOT AMCOLNAM_HOST_ENAB AMSQL_IS_ENABLED AMSQL_AND \
	AMTABNAM_HOST AMSQL_DOT AMCOLNAM_HOST_NAME AMSQL_READ_STRING AMSQL_PTHE

#define AMSQL_HOST_HOID	\
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_HOTY AMSQL_READ_INTEGER AMSQL_AND \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_HOID AMSQL_BLANK AMSQL_IN AMSQL_PTHS \
	AMSQL_SELECT AMCOLNAM_HOST_ID AMSQL_BLANK AMSQL_FROM AMTABNAM_HOST AMSQL_BLANK AMSQL_WHERE \
	AMCOLNAM_HOST_ENAB AMSQL_IS_ENABLED AMSQL_AND \
	AMCOLNAM_HOST_NAME AMSQL_READ_STRING AMSQL_PTHE

#define AMSQL_GUEST_GRUP \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_GUTY AMSQL_READ_INTEGER AMSQL_AND \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_GUID AMSQL_EQUAL \
	AMTABNAM_GRUP AMSQL_DOT AMCOLNAM_GRUP_ID AMSQL_BLANK AMSQL_AND \
	AMTABNAM_GRUP AMSQL_DOT AMCOLNAM_GRUP_ENAB AMSQL_IS_ENABLED AMSQL_AND \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_GUID AMSQL_BLANK AMSQL_IN AMSQL_PTHS \
	AMSQL_SELECT AMTABNAM_USGR AMSQL_DOT AMCOLNAM_USGR_GRID AMSQL_BLANK AMSQL_FROM \
	AMTABNAM_USGR AMSQL_COMMA AMTABNAM_USER AMSQL_BLANK AMSQL_WHERE \
	AMTABNAM_USGR AMSQL_DOT AMCOLNAM_USGR_USID AMSQL_EQUAL \
	AMTABNAM_USER AMSQL_DOT AMCOLNAM_USER_ID AMSQL_BLANK AMSQL_AND \
	AMTABNAM_USER AMSQL_DOT AMCOLNAM_USER_ENAB AMSQL_IS_ENABLED AMSQL_AND \
	AMTABNAM_USER AMSQL_DOT AMCOLNAM_USER_NAME AMSQL_READ_STRING AMSQL_PTHE

#define AMSQL_GUEST_USER \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_GUTY AMSQL_READ_INTEGER AMSQL_AND \
	AMTABNAM_MAIN AMSQL_DOT AMCOLNAM_MAIN_GUID AMSQL_BLANK AMSQL_IN AMSQL_PTHS \
	AMSQL_SELECT AMCOLNAM_USER_ID AMSQL_BLANK AMSQL_FROM AMTABNAM_USER AMSQL_BLANK AMSQL_WHERE \
	AMCOLNAM_USER_ENAB AMSQL_IS_ENABLED AMSQL_AND \
	AMCOLNAM_USER_NAME AMSQL_READ_STRING AMSQL_PTHE

#define AMSQL_GUEST_ANYB _T("DWMAIN.GUESTTYPE=") AM_ANYB_T
#define AMSQL_HOST_ANYB  _T("DWMAIN.HOSTTYPE=") AM_ANYB_T

typedef struct _AMVALS
{
	U32 type;
	U32 width;
	void *value; // A TCHAR pointer which also can be used to store integer numbers

} AMVALS;

typedef struct _ROLE
{
	TCHAR name[AM_NAMLEN];
	TCHAR desc[AM_DESCRLEN];
	TCHAR created[AM_DATELEN];
	TCHAR createdBy[AM_DBUSRLEN];
	TCHAR modified[AM_DATELEN];
	TCHAR modifiedBy[AM_DBUSRLEN];
	AMBOOL Ctrl;
	AMBOOL Keyb;
	AMBOOL Lock;
	AMBOOL Blnk;
	AMBOOL Boot;
	AMBOOL Clip;
	AMBOOL Send;
	AMBOOL Recv;
	AMBOOL Prnt;
	AMBOOL Chat;
	AMBOOL Audi;
	AMBOOL Runp;
	U32 id;
	U32 Conf;
} ROLE;

typedef struct _NUSER
{
	TCHAR name[AM_NAMLEN];
	TCHAR desc[AM_DESCRLEN];
	TCHAR password[AM_PWDLEN];
	TCHAR p0[AM_PWDLEN];
	TCHAR p1[AM_PWDLEN];
	TCHAR p2[AM_PWDLEN];
	TCHAR p3[AM_PWDLEN];
	TCHAR p4[AM_PWDLEN];
	TCHAR p5[AM_PWDLEN];
	TCHAR p6[AM_PWDLEN];
	TCHAR p7[AM_PWDLEN];
	TCHAR p8[AM_PWDLEN];
	TCHAR p9[AM_PWDLEN];
	TCHAR lastused[AM_DATELEN];
	TCHAR lastchanged[AM_DATELEN];
	TCHAR callback[AM_CALLBACKLEN];
	TCHAR created[AM_DATELEN];
	TCHAR createdBy[AM_DBUSRLEN];
	TCHAR modified[AM_DATELEN];
	TCHAR modifiedBy[AM_DBUSRLEN];
	U32 id;
	U32 enabled;
	U32 mustBeChanged;
	U32 nIllegal;
	U32 nOld;
	U32 mode;
} NUSER;

typedef struct _NHOST
{
	TCHAR name[AM_NAMLEN];
	TCHAR desc[AM_DESCRLEN];
	TCHAR created[AM_DATELEN];
	TCHAR createdBy[AM_DBUSRLEN];
	TCHAR modified[AM_DATELEN];
	TCHAR modifiedBy[AM_DBUSRLEN];
	U32 id;
	U32 enabled;
} NHOST;

typedef struct _NGROUP
{
	// TCHAR sid[AM_SIDLEN];
	TCHAR name[AM_NAMLEN];
	TCHAR desc[AM_DESCRLEN];
	TCHAR created[AM_DATELEN];
	TCHAR createdBy[AM_DBUSRLEN];
	TCHAR modified[AM_DATELEN];
	TCHAR modifiedBy[AM_DBUSRLEN];
	U32 id;
	U32 enabled;
	U32 type;
} NGROUP;

typedef struct _NTUSER
{
	TCHAR name[AM_UNLEN];
	TCHAR domain[AM_UNLEN];
	TCHAR created[AM_DATELEN];
	TCHAR createdBy[AM_DBUSRLEN];
	TCHAR modified[AM_DATELEN];
	TCHAR modifiedBy[AM_DBUSRLEN];
	U32 id;
	U32 rid;
	U32 enabled;
} NTUSER;

typedef struct _NTGROUP
{
	TCHAR name[AM_UNLEN];
	TCHAR domain[AM_UNLEN];
	TCHAR created[AM_DATELEN];
	TCHAR createdBy[AM_DBUSRLEN];
	TCHAR modified[AM_DATELEN];
	TCHAR modifiedBy[AM_DBUSRLEN];
	U32 id;
	U32 rid;
	U32 enabled;
} NTGROUP;

typedef struct _AMDOMAIN
{
	TCHAR name[AM_UNLEN];
	TCHAR created[AM_DATELEN];
	TCHAR createdBy[AM_DBUSRLEN];
	TCHAR modified[AM_DATELEN];
	TCHAR modifiedBy[AM_DBUSRLEN];
	U32 id;
	U32 enabled;
} AMDOMAIN;

typedef struct _AMSERVER
{
	TCHAR name[AM_UNLEN];
	TCHAR akey[AM_NAMLEN];
	TCHAR created[AM_DATELEN];
	TCHAR createdBy[AM_DBUSRLEN];
	TCHAR modified[AM_DATELEN];
	TCHAR modifiedBy[AM_DBUSRLEN];
 	U32 isRunning; // 0 = not running, 1 = running
	U32 type; // 0 = security server, 1 = access server, 999 = group name
} AMSERVER;

typedef struct _WKST
{
	TCHAR name[AM_UNLEN];
	TCHAR domain[AM_UNLEN];
	TCHAR created[AM_DATELEN];
	TCHAR createdBy[AM_DBUSRLEN];
	TCHAR modified[AM_DATELEN];
	TCHAR modifiedBy[AM_DBUSRLEN];
	U32 id;
	U32 enabled;
} WKST;

typedef struct _MAIN
{
	TCHAR gname[AM_PARAMLEN];
	TCHAR hname[AM_PARAMLEN];
	TCHAR rname[AM_NAMLEN];
	TCHAR created[AM_DATELEN];
	TCHAR createdBy[AM_DBUSRLEN];
	TCHAR modified[AM_DATELEN];
	TCHAR modifiedBy[AM_DBUSRLEN];
	U32 id;
	U32 enabled;
	U32 gtyp;
	U32 gid;
	U32 htyp;
	U32 hid;
	U32 rid;
} MAIN;

typedef struct _CONN
{
	TCHAR guest[AM_PARAMLEN];
	TCHAR host[AM_PARAMLEN];
	TCHAR started[AM_DATELEN];
	U32 type;
} CONN;

typedef struct _LOG
{
	TCHAR p1[AM_PARAMLEN];
//	TCHAR p2[AM_PARAMLEN];
//	TCHAR p3[AM_PARAMLEN];
	TCHAR created[AM_DATELEN];
	TCHAR createdBy[AM_DBUSRLEN];
	U32 id;
	U32 status;
	U32 action;
	U32 operand;
	U32 operat;
} LOG;

typedef struct _SCHEDULE
{
	TCHAR p1[AM_PARAMLEN];
	TCHAR p2[AM_PARAMLEN];
	TCHAR created[AM_DATELEN];
	TCHAR createdBy[AM_DBUSRLEN];
	TCHAR executeAt[AM_DATELEN];
	U32 id;
	U32 batch;
	U32 action;
	U32 operand;
	U32 operat;
} SCHEDULE;

typedef struct _BATCH
{
	TCHAR domain[AM_UNLEN];
	TCHAR desc[AM_DESCRLEN];
	TCHAR start[AM_DATELEN];
	TCHAR end[AM_DATELEN];
	TCHAR created[AM_DATELEN];
	TCHAR createdBy[AM_DBUSRLEN];
	TCHAR modified[AM_DATELEN];
	TCHAR modifiedBy[AM_DBUSRLEN];
	U32 groupID;
	U32 id;
	U32 type;
	U32 flags;
} BATCH;

typedef struct _AMEVNT
{
	TCHAR desc[AM_TXTLEN];
	TCHAR host[AM_NAMLEN];
	TCHAR date[AM_DATELEN];
	TCHAR event[AM_CMDLEN];
	U32 serial;
	U32 dtlError;
	U32 protError;
	U32 id;
} AMEVNT;

typedef struct _AM_PWDOPTIONS
{
	U32 minlength;
	U32 numpwd;
	U32 lifetime;
	U32 lock_after;
	U32 lock_ifnotused;
	AMBOOL restricted;
} AM_PWDOPTIONS;

typedef struct _AM_LOGOPTIONS
{
	U32 deleteOlderThan;
} AM_LOGOPTIONS;

// Sorry boys, but including DTL2.H here would drag in .......
#define AS_INSTALL_KEYBOARD        0x00000004
#define AS_INSTALL_MOUSE           0x00000008
#define AS_INSTALL_CHAT            0x00000010
#define AS_INSTALL_KBDLOCK         0x00000020
#define AS_INSTALL_BOOT            0x00000040
#define AS_INSTALL_DISPBLANK       0x00000080
#define AS_INSTALL_UPLOADFILES     0x00000100
#define AS_INSTALL_DOWNLOADFILES   0x00000200
#define AS_INSTALL_CONFACCESS      0x00000400   // 990607
#define AS_INSTALL_REMOTE_CTRL     0x00004000   // 980917 new
#define AS_INSTALL_AUDIO_CHAT      0x02000000ul   // 980917 new  990609

#define AS_FLMX_CLIPBOARD           0x00000008  // 990607
#define AS_FLMX_REMOTE_PRINT        0x00000010  // 990607
#define AS_FLMX_CONFACCESS_IGNORE   0x00000020  // 990607
#define AS_FLMX_RUN_PROGRAM         0x00000800  // 990607

typedef struct {
   U32                  FeatureList;
   U32                  FeatureListOr;          // 990607
   U32                  FeatureListEx;          // 990607
   U32                  FeatureListExOr;        // 990607
   } AM_CH_ACCESS;
typedef AM_CH_ACCESS *PAM_CH_ACCESS;
typedef AM_CH_ACCESS DB_FEATURELIST;

#else
#endif